/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

package com.fundi.framework.common.change;

import java.util.LinkedHashMap;
import java.util.Map;

import com.fundi.framework.common.displaylist.ListField;
import com.fundi.framework.common.displaylist.ListFieldDefn;
import com.fundi.framework.common.displaylist.PropDefn;

public class PropDefnChangeableWrapper implements IChangeable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$

	private	Map<String, PropDefn>	source;
	private ChangeListScope			cls;
	private final boolean			forceNoInitialValue;
	
	public PropDefnChangeableWrapper(Map<String, PropDefn> definitions) {
		this(definitions, false);
	}
		
	public PropDefnChangeableWrapper(Map<String, PropDefn> definitions, boolean forceNoInitialValue) {
		this.source = definitions;
		this.forceNoInitialValue = forceNoInitialValue;
	}

	@Override
	public Map<String, PropDefn> getSource() {
		return source;
	}

	@Override
	public ChangeListScope getChangeListScope() {
		return cls;
	}

	@Override
	public void setChangeListScope(ChangeListScope cls) {
		this.cls = cls;
	}

	@Override
	public boolean isRecognisedKey(Object key) {
		return source.containsKey(key);
	}

	@Override
	public String getInitialValue(Object key) {
		PropDefn	pdefn;
		
		pdefn = source.get(key);
		
		if (forceNoInitialValue) {
			return null;
		} else {
			return (String) (pdefn != null ? source.get(key).getDefaultValue() : ""); //$NON-NLS-1$
		}
	}

	@Override
	public String getFinalValue(Object key) {
		return (String) cls.getFinalValue(source, key);	//convenience method really
	}

	@Override
	public ListField getInitialObject(Object key) {
		ListField	field;
		PropDefn	pdefn;
		
		field = null;
		if (source.containsKey(key)) {
			pdefn = source.get(key);
			field = new ListField(new ListFieldDefn(pdefn), (forceNoInitialValue ? null : pdefn.getDefaultValue()));
		}
		
		return field;
	}

	@Override
	public ListField getFinalObject(Object key) {
		ListField field;
		
		field = (ListField) getInitialObject(key);
		field.setValue((String) getFinalValue(key));
		
		return field;
	}

	@Override
	public Map<?, ?> getChangedObjects() {
		Map<Object, Object>	changes;
		Map<String, ListField>	changedObjects;
		
		changedObjects = new LinkedHashMap<String, ListField>();
		changes = cls.getValues(); 
		for (Object key: changes.keySet()) {
			changedObjects.put((String) key, getFinalObject(key));
		}
		
		return changedObjects;
	}

}
