package com.fundi.framework.common.displaylist;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.StringWriter;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.util.StringEditor;

@XmlRootElement
public class ColumnLayout {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$

	String		label;
	Integer		maxWidth = null;
	boolean		visible = true;
	boolean		hiddenByServer = false;
	
	public ColumnLayout() {
		super();
	}
	
	public ColumnLayout(String label) throws IllegalArgumentException {
		if (label == null || label.isEmpty()) throw new IllegalArgumentException(Messages.getString("ColumnLayout_0")); //$NON-NLS-1$

		this.label = label;
	}

	public ColumnLayout(ListFieldDefn defn) throws IllegalArgumentException {
		apply(defn);
	}
	
	public void apply(ListFieldDefn defn) throws IllegalArgumentException {	
		if (defn == null || defn.getLabel() == null && defn.getLabel().isEmpty()) throw new IllegalArgumentException(Messages.getString("ColumnLayout_1")); //$NON-NLS-1$
		if (label != null && !label.isEmpty() && !label.equals(defn.getLabel())) throw new IllegalArgumentException(Messages.getString("ColumnLayout_2")); //$NON-NLS-1$
		
		this.label = defn.getLabel();
		this.visible = !defn.isHiddenByUser();
		this.hiddenByServer = defn.isHiddenByServer();
	}

	public String getLabel() {
		return label;
	}
	
	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public void setLabel(String label) {
		this.label = label;
	}

	public Integer getMaxWidth() {
		return maxWidth;
	}

	public void setMaxWidth(Integer maxWidth) {
		this.maxWidth = maxWidth;
	}

	@Override
	public String toString() {
		JAXBContext		xmlContext;
		Marshaller		m;
		StringWriter 	writer;
		String			result;
		
		result = null;
		try {
			xmlContext = JAXBContext.newInstance(ColumnLayout.class);
			m = xmlContext.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
			writer = new StringWriter();
			m.marshal(this, writer);
			result = writer.toString();
			writer.close();
		} catch (Throwable e) {
			e.printStackTrace();
		}
		
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		boolean			result;
		ColumnLayout	other;
		
		result = false;
		if (ColumnLayout.class.isInstance(obj)) {
			other = (ColumnLayout) obj;
			result = StringEditor.bothNullOrEqual(this.getLabel(), other.getLabel())
				&& this.isVisible() == other.isVisible()
			;
		}
		
		return result;
	}

	public boolean isHiddenByServer() {
		return hiddenByServer;
	}

	public void setHiddenByServer(boolean hiddenByServer) {
		this.hiddenByServer = hiddenByServer;
	}

}
